using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Threading;
using System.Windows;
using System.Windows.Threading;

namespace DataGridCrash
{
    /// <summary>
    /// Class containing the business logic used to interact with Car data.  In this sample
    /// the class only contains one method which returns a collection of cars, but in 
    /// the real-world you would probably also have methods for finding an individual car,
    /// saving car data, or accessing a collection of cars based on some filter criteria.
    /// </summary>
    public class CarsBusinessLogic
    {
        readonly ObservableCollection<Car> _cars = new ObservableCollection<Car>();

        public ObservableCollection<Car> Cars
        {
            get { return _cars; }
        }

        public CarsBusinessLogic()
        {
            PopulateCars();
        }

        public void PopulateCars()
        {
            _cars.Clear();
            for (int i = 0; i < 250; i++)
            {
                List<Car> cars = new List<Car>
                    {
                        new Car("Dodge", "Ram", Color.Blue, 22050.00, 153),
                        new Car("Ford", "Explorer", Color.Green, 27175.00, 96),
                        new Car("BMW", "Z4", Color.Silver, 35600.00, 42),
                        new Car("Toyota", "Camry", Color.Black, 20790.99, 131)
                    };
                foreach (var car in cars)
                {
                    _cars.Add(car);
                }
            }
        }

        public void DeleteCars()
        {
            int nToRemove = _cars.Count >> 1;

            for (int i = 0; i < nToRemove; i++)
            {
                _cars.RemoveAt(0);
                //Thread.Sleep(1);
            }

            Application.Current.Dispatcher.BeginInvoke((Action)(() =>
                {
                    Thread.Sleep(10);
                    try
                    {
                        _cars.Clear();
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("DeleteCars:\n" + e);
                        MessageBox.Show(e.ToString(), "Here We Are!");
                    }
                }),
                DispatcherPriority.Normal);
        }
    }

    /// <summary>
    /// The sample Car class contains information about a specific car
    /// </summary>
    public class Car : INotifyPropertyChanged
    {
        string m_make;
        string m_model;
        Color m_color;
        double m_baseprice;
        int m_milage;

        public Car(string make, string model, Color color, double baseprice, int milage)
        {
            Make = make;
            Model = model;
            Color = color;
            BasePrice = baseprice;
            Milage = milage;
        }

        public string Make
        {
            get { return m_make; }
            set
            {
                if (m_make != value)
                {
                    m_make = value;
                    NotifyPropertyChanged("Make");
                }
            }
        }

        public string Model
        {
            get { return m_model; }
            set
            {
                if (m_model != value)
                {
                    m_model = value;
                    NotifyPropertyChanged("Model");
                }
            }
        }

        public Color Color
        {
            get { return m_color; }
            set
            {
                if (m_color != value)
                {
                    m_color = value;
                    NotifyPropertyChanged("Color");
                }
            }
        }

        public double BasePrice
        {
            get { return m_baseprice; }
            set
            {
                if (m_baseprice != value)
                {
                    m_baseprice = value;
                    NotifyPropertyChanged("BasePrice");
                }
            }
        }

        public int Milage
        {
            get { return m_milage; }
            set
            {
                if (m_milage != value)
                {
                    m_milage = value;
                    NotifyPropertyChanged("Milage");
                }
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }
}
